// Terrain Build Rules
// A script that enforces brick placement rules for a community terrain build

// This bit is here to make re-executing the script easier
if(isPackage(TerrainBuildRulesPackage))
   deactivatePackage(TerrainBuildRulesPackage);

package TerrainBuildRulesPackage
{
   function ServerCmdPlantBrick(%client)
   {
      // Call the default version of the plant function
      // Compare the player's brick count before and after to see if plant succeeded

      %player = %client.player;
      if(!isObject(%player))
         return 0;

      %tempBrick = %player.tempBrick;
      if(!isObject(%tempBrick))
         return 0;

      %preCount = %client.brickGroup.getCount();
      Parent::ServerCmdPlantBrick(%client);
      %count = %client.brickGroup.getCount();
      if(%count <= %preCount)
         return;
      
      // The last brick in their brick group should be the one they just planted
      %plantedBrick = %client.brickGroup.getObject(%count - 1);
      
      // Special rules for the first brick planted, so we can get the build started
      // have to do some messed up stuff here because temp bricks count towards brickcount now
      // this can still be broken if people are spamming non 64x bricks because of the delay in deleting
      if(getBrickCount() <= 40)
      {
         %tempBrickCount = 0;
         for(%i = 0; %i < clientGroup.getCount(); %i++)
         {
            %cl = clientGroup.getObject(%i);

            for(%j = 0; %j < %cl.brickGroup.getcount(); %j++)
            {
               %brick = %cl.brickGroup.getObject(%j);
               if(%brick.isDead())
                  %tempBrickcount++;
            }

            if(isObject(%cl.player))
               if(isObject(%cl.player.tempBrick))
                  %tempBrickCount++;            
         }

         if(getBrickCount() - %tempBrickCount <= 1)
         {
            // if they managed to plant the first brick not on the ground then something is wrong
            if(%plantedBrick.getDistanceFromGround() != 0)
            {
               error("ERROR: [TerrainBuildRulesPackage]ServerCmdPlantBrick() - first brick somehow not planted on the ground");
               return;
            }
            
            // must be a 64x plate
            if(%plantedBrick.getDataBlock().getID() != brick64x64fData.getID())
            {
               CommandToClient(%client, 'MessageBoxOK', "Invalid Brick", "You must start with a 64x64 Base<bitmap:base/client/ui/brickIcons/64x64 Base>");
               %plantedBrick.killBrick();
               return;
            }
            
            return;
         }
      }
     
      // They planted a brick.  Was it on the ground?
      if(%plantedBrick.getDistanceFromGround() != 0)
         return;

      // It was on the ground.  It better be a 64x baseplate.
      if(%plantedBrick.getDataBlock().getID() != brick64x64fData.getID())          //Note: comparing the two datablock ID numbers is faster than doing a string compare on their names
      {
         CommandToClient(%client, 'MessageBoxOK', "Invalid Brick", "You must start with a 64x64 Base<bitmap:base/client/ui/brickIcons/64x64 Base>");
         %plantedBrick.killBrick();
         return;
      }

      // Also, it better be lined up with an existing baseplate
      // Search for nearby plates and check their relative position
      %pos = %plantedBrick.getPosition();
      %box = "32.1 32.1 0.1";
      %mask = $TypeMasks::FxBrickAlwaysObjectType;

      %plantX = getWord(%plantedBrick.getPosition(), 0);
      %plantY = getWord(%plantedBrick.getPosition(), 1);

      initContainerBoxSearch(%pos, %box, %mask);
	   while (%searchObj = containerSearchNext())
	   {	
         if(%searchObj.getDataBlock().getID() != brick64x64fData.getID())
            continue;

         if(%searchObj.getDistanceFromGround() != 0)
            continue;

         if(!%searchObj.isPlanted())
            continue;
         
         if(%searchObj.isDead())
            continue;

         if(%searchObj == %plantedBrick)
            continue;

         //To be lined up, the x or y position must be the same
         %searchX = getWord(%searchObj.getPosition(), 0);
         %searchY = getWord(%searchObj.getPosition(), 1);

         if(mAbs(%searchX - %plantX) < 0.05 ||               // Note: We compare with subtraction like this because direct comparison with "==" on floating point numbers is not reliable
            mAbs(%searchY - %plantY) < 0.05 )
            return;
      }
      
      //If we got this far it must not have been lined up with any existing plates
      CommandToClient(%client, 'MessageBoxOK', "Invalid Placement", "You must line up your baseplate with the existing grid.");
      %plantedBrick.killBrick();
   }
};

activatePackage(TerrainBuildRulesPackage);